@@ -5,7 +5,7 @@
 # Author    : Chris Jones
 # Email     : crj341@student.bham.ac.uk
 # Date      : 26/06/2020
-# Version   : 0.3.2
+# Version   : 0.3.3
 
 import  numpy                   as      np
 import  plotly.graph_objects    as      go
@@ -197,9 +197,16 @@ class AFM:
         self.z_position_raw = []
         self.z_approach_raw = []
         self.z_withdraw_raw = []
+        self.z_approach_piezo =[]
+        self.z_withdraw_piezo =[]
+
 
         self.adhesion = []
         self.modulus = []
+        self.contact_point = []
+        self.indent_depth = []
+        self.snap_in = []
+        self.max_force = []
 
         self.data_offset = []
         self.data_length = []
@@ -612,7 +619,11 @@ class AFM:
         Function to align the contact point of the approach curve with zero by
         finding the first point to cross the axis and interpolating between this
         and the previous value. Note this method may be unstable for noisy curves.
+        The tip contact height and maximum indentation depth are also saved.
         '''
+
+        self.contact_point = np.zeros(self.z_approach.shape[1])
+        self.indent_depth = np.zeros(self.z_approach.shape[1])
         
         for i in range(0, self.z_approach.shape[1]):
             _approach = self.approach[:,i]
@@ -634,6 +645,8 @@ class AFM:
                 if np.isnan(intercept) == False:
                     self.z_approach[:,i] = self.z_approach[:,i] - intercept
                     self.z_withdraw[:,i] = self.z_withdraw[:,i] - intercept
+                    self.contact_point[i] = intercept
+                    self.indent_depth[i] = next(n for n in self.z_approach[:,i] if n == n)
 
             except:
                 print('WARNING: Could not locate contact point for curve number ', i, '. Consider checking baseline alignment.')
@@ -641,6 +654,12 @@ class AFM:
     def adjust_tip_position(
         self
     ):
+        '''
+        Convert piezo position to actual tip position.
+        '''
+
+        self.z_approach_piezo = self.z_approach
+        self.z_withdraw_piezo = self.z_withdraw
 
         for i in range(0, self.z_approach.shape[1]):
 
@@ -760,6 +779,37 @@ class AFM:
             except:
                 pass
 
+            try:
+                self.z_approach_piezo = np.delete(self.z_approach_piezo, curve_int, axis = 1)
+            except:
+                pass
+
+            try:
+                self.z_withdraw_piezo = np.delete(self.z_withdraw_piezo, curve_int, axis = 1)
+            except:
+                pass
+
+            try:
+                self.contact_point = np.delete(self.contact_point, curve_int, axis = 1)
+            except:
+                pass
+
+            try:
+                self.indent_depth = np.delete(self.indent_depth, curve_int, axis = 1)
+            except:
+                pass
+
+            try:
+                self.snap_in = np.delete(self.snap_in, curve_int, axis = 1)
+            except:
+                pass
+
+            try:
+                self.max_force = np.delete(self.max_force, curve_int, axis = 1)
+            except:
+                pass
+            
+
             i += 1
             for j in range(i, len(args)):
                 if curves[j] > curve_number:
@@ -785,6 +835,8 @@ class AFM:
             retract = self.retract,
             z_approach = self.z_approach,
             z_withdraw = self.z_withdraw,
+            z_approach_piezo = self.z_approach_piezo,
+            z_withdraw_piezo = self.z_withdraw_piezo,
             adhesion = self.adhesion,
             modulus = self.modulus,
             data_offset = self.data_offset,
@@ -797,7 +849,11 @@ class AFM:
             spr_const = self.spr_const,
             file_name = self.file_name,
             run_name = self.run_name,
-            measurements = self.measurements
+            measurements = self.measurements,
+            contact_point = self.contact_point,
+            indent_depth = self.indent_depth,
+            snap_in = self.snap_in,
+            max_force = self.max_force
         )
 
     def load_data(
@@ -833,6 +889,12 @@ class AFM:
         self.file_name = data['file_name']
         self.run_name = data['run_name']
         self.measurements = data['measurements']
+        self.z_approach_piezo = data['z_approach_piezo']
+        self.z_withdraw_piezo = data['z_withdraw_piezo']
+        self.contact_point = data['contact_point']
+        self.indent_depth = data['indent_depth']
+        self.snap_in = data['snap_in']
+        self.max_force = data['max_force']
 
     def calc_adhesion(
         self,
@@ -841,13 +903,18 @@ class AFM:
         '''
         Funciton to calculate the adhesion value for each curve by finding the minimum
         point on the retact curve. A histogram of calculated values can be plotted by
-        setting the 'plot_hist' input to 'True'
+        setting the 'plot_hist' input to 'True'. The snap in force and maximum indentation
+        force are also saved.
         '''
 
         self.adhesion = np.zeros(self.z_approach.shape[1])
+        self.snap_in = np.zeros(self.z_approach.shape[1])
+        self.max_force = np.zeros(self.z_approach.shape[1])
 
         for curve_number in range(0, self.z_approach.shape[1]):
             self.adhesion[curve_number] = np.nanmin(self.retract[:, curve_number]) * -1
+            self.snap_in[curve_number] = np.nanmin(self.approach[:, curve_number]) * -1
+            self.max_force[curve_number] = np.nanmax(self.approach[:, curve_number])
 
         if plot_hist is True:
             fig = go.Figure(data = [go.Histogram(x = self.adhesion)])
@@ -1231,7 +1298,8 @@ class AFM:
         x_cells = 10,
         y_cells = 10,
         x_scale = 10,
-        y_scale = 10
+        y_scale = 10,
+        return_fig = False
     ):
 
         x_pos = np.zeros(x_cells * y_cells)
@@ -1268,8 +1336,6 @@ class AFM:
                 x_pos[j] = x_counter
                 y_pos[j] = y_counter
                 adhesion[j] = self.adhesion[j]
-                modulus[j] = self.modulus[j]
-                depth[j]= next(n for n in self.z_approach[:,j] if n == n)
 
                 if updown == 1:
                     updown = 2
@@ -1328,4 +1394,7 @@ class AFM:
             )
         )
 
-        fig.show()
\ No newline at end of file
+        fig.show()
+
+        if return_fig == True:
+            return fig
\ No newline at end of file
